# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import os
from typing import List
from pydantic import BaseModel
from aliyunsdkcore.auth.credentials import RamRoleArnCredential
from tencentcloud.common.credential import STSAssumeRoleCredential


class DbConfig(BaseModel):
    host: str = os.getenv('CLOUD_ASSERT_HOST', '')
    port: int = os.getenv('CLOUD_ASSERT_PORT', 5432)
    password: str = os.getenv('CLOUD_ASSERT_PASSWORD', '')
    user: str = os.getenv('CLOUD_ASSERT_USER', '')
    database: str = os.getenv('CLOUD_ASSERT_DATABASE', '')


class AssetColumn(BaseModel):
    name: str
    type: str = 'str'
    length: int = None
    kwargs: dict = {'nullable': True, 'default': None}


class TencentRole(BaseModel):
    arn: str
    session_name: str = 'fetch_asset'
    duration_seconds: int = 3600


class AliyunRole(BaseModel):
    arn: str
    session_name: str = 'fetch_asset'


class AwsRole(AliyunRole):
    duration_seconds: int = 3600


class TencentProfile(BaseModel):
    ak: str
    sk: str
    roles: List[TencentRole] = []


class AliyunProfile(BaseModel):
    ak: str
    sk: str
    roles: List[AliyunRole] = []


class AwsProfile(BaseModel):
    roles: List[AwsRole] = []


class AwsCredential(BaseModel):
    aws_access_key_id: str
    aws_secret_access_key: str
    aws_session_token: str


class HuaweiCredential(BaseModel):
    ak: str
    sk: str


class HuaweiProfile(BaseModel):
    credentials: List[HuaweiCredential]






# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

from setuptools import setup


packages = [
    'asset',
    'asset.aliyun',
    'asset.tencent',
    'asset.aws',
    'asset.huawei'
]
install_requires = [
    'tencentcloud-sdk-python==3.0.898',

    'aliyun-python-sdk-core==2.13.36',
    'aliyun-python-sdk-ecs==4.24.26',
    'aliyun-python-sdk-sts==3.1.0',
    'aliyun-python-sdk-ram==3.3.0',

    'boto3==1.26.29',

    'huaweicloudsdkcore==3.1.37',
    'huaweicloudsdkecs==3.1.37',
    'huaweicloudsdkiam==3.1.37',
    'huaweicloudsdkevs==3.1.37',
    'huaweicloudsdkeip==3.1.37',
    'huaweicloudsdknat==3.1.37',
    'huaweicloudsdkelb==3.1.37',
    'huaweicloudsdkvpc==3.1.37',
    'esdk-obs-python==3.22.2',

    'SQLAlchemy==1.4.44',
    'psycopg2==2.9.5',
    'click==8.1.3',
    'pydantic==1.10.2',
    'requests==2.28.0',
    'pyyaml==6.0',
    'arrow==1.2.3'
]
entry_points = {'console_scripts': ['cloud-asset = asset.cli:main']}
setup_kwargs = {
    'name': 'cloud-asset',
    'version': '0.0.5.0',
    'description': 'Cloud Asset - Asset For Postgres',
    'license': 'Apache-2.0',
    'long_description': '',
    'long_description_content_type': 'text/markdown',
    'author': 'Cloud Asset Authors',
    'author_email': '',
    'packages': packages,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<4.0',
}

setup(**setup_kwargs)



# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0


class AssetRegister:
    def __init__(self):
        self._assets = {}

    @property
    def assets(self) -> dict:
        return self._assets

    def register(self, name: str):
        def _register_class(asset):
            self._assets[name] = asset
            return asset
        return _register_class


class TencentAssetRegister(AssetRegister):
    pass


class AliyunAssetRegister(AssetRegister):
    pass


class AwsAssetRegister(AssetRegister):
    pass


class HuaweiRegister(AssetRegister):
    pass


class CloudProviders:
    tencent = TencentAssetRegister()
    aliyun = AliyunAssetRegister()
    aws = AwsAssetRegister()
    huawei = HuaweiRegister()


cloud_providers = CloudProviders()







# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import datetime


def asset_columns_generator(fields: dict):
    _fields = []
    for key, value in fields.items():
        if isinstance(value, str) or isinstance(value, bool):
            _type = 'str'
        elif isinstance(value, int):
            _type = 'int'
        elif isinstance(value, dict) or isinstance(value, list):
            _type = 'dict'
        elif isinstance(value, datetime.date):
            _type = 'date'
        elif isinstance(value, datetime.datetime):
            _type = 'datetime'
        elif isinstance(value, float):
            _type = 'float'
        else:
            raise Exception(f'不支持的类型, value: {value}')
        _fields.append({'name': key, 'type': _type})
    return _fields


if __name__ == '__main__':

    fields_demo = {
                "AddTime": "2020-11-28 10:05:52",
                "ConsoleLogin": 0,
                "DeletionTaskId": 'null',
                "Description": "当前角色为腾讯云自动化助手 (TAT)服务相关角色，该角色将在已关联策略的权限范围内访问您的其他云服务资源。",
                "PolicyDocument": "{\"version\":\"2.0\",\"statement\":[{\"action\":\"sts:AssumeRole\",\"effect\":\"allow\",\"principal\":{\"service\":[\"command.tat.cloud.tencent.com\"]}}]}",
                "RoleId": "4611686018433460000",
                "RoleName": "TAT_QCSLinkedRoleInCommand",
                "RoleType": "service_linked",
                "SessionDuration": 43200,
                "Tags": [],
                "UpdateTime": "2020-11-28 10:05:52"
    }
    for asset in asset_columns_generator(fields=fields_demo):
        print(str(asset) + ',')


# -*- coding: UTF-8 -*-

# Copyright The Cloud Asset Authors.
# SPDX-License-Identifier: Apache-2.0

import copy

from asset.asset_register import cloud_providers
from asset.base import AwsAsset, AssetColumn, UniqueConstraint

field_document = 'https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/elb.html#ElasticLoadBalancing.Client.describe_load_balancers'


asset_columns = [
    {'name': 'LoadBalancerName', 'type': 'str'},
    {'name': 'DNSName', 'type': 'str'},
    {'name': 'CanonicalHostedZoneName', 'type': 'str'},
    {'name': 'CanonicalHostedZoneNameID', 'type': 'str'},
    {'name': 'ListenerDescriptions', 'type': 'dict'},
    {'name': 'Policies', 'type': 'dict'},
    {'name': 'BackendServerDescriptions', 'type': 'dict'},
    {'name': 'AvailabilityZones', 'type': 'dict'},
    {'name': 'Subnets', 'type': 'dict'},
    {'name': 'VPCId', 'type': 'str'},
    {'name': 'Instances', 'type': 'dict'},
    {'name': 'HealthCheck', 'type': 'dict'},
    {'name': 'SourceSecurityGroup', 'type': 'dict'},
    {'name': 'SecurityGroups', 'type': 'dict'},
    {'name': 'CreatedTime', 'type': 'date'},
    {'name': 'Scheme', 'type': 'str'},
    {'name': 'Tags', 'type': 'dict'}
]


@cloud_providers.aws.register('elb')
class Elb(AwsAsset):
    _client_name = 'elb'
    _des_request = 'describe_load_balancers'
    _response_field = 'LoadBalancerDescriptions'
    _des_request_kwargs: dict = {'PageSize': 400}

    _table_name = 'aws_elb'
    _asset_columns = [AssetColumn(**asset_column) for asset_column in asset_columns]
    _table_args = (UniqueConstraint('account_id', 'record_date', 'load_balancer_name', name='aws_uc_elb'),)
    _field_document = field_document

    def _paginate_all_assets(self) -> list:
        tags = []
        _des_request_kwargs = copy.deepcopy(self._des_request_kwargs)
        _assets, _ = self._describe(
            self.client,
            self._des_request,
            self._response_field,
            self._child_response_filed,
            des_request_kwargs=_des_request_kwargs,
            parser_response_func=self.parser_response
        ).parser_response()

        for index in range(len(_assets) // 20 + 1):
            """get tags"""
            elb_list = _assets[index: 20: index * 20 + 20]
            if not elb_list:
                break
            _tags, _ = self._describe(
                self.client,
                'describe_tags',
                'TagDescriptions',
                des_request_kwargs={'LoadBalancerNames': [elb['LoadBalancerName'] for elb in elb_list]},
                parser_response_func=self.parser_response
            ).parser_response()
            tags += _tags
        if tags:
            for asset in _assets:
                try:
                    tag = filter(lambda x: x['LoadBalancerName'] == asset['LoadBalancerName'], tags).__next__()
                except Exception as e:
                    tag = []
                asset.update({'Tags': tag})
        return _assets




